#include <VRS51L3074_SDCC.h>

#include "SHTxx.h"


//----------------------------------------------------------------------------------
char s_write_byte(unsigned char value)
//----------------------------------------------------------------------------------
// writes a byte on the Sensibus and checks the acknowledge 
{ 
unsigned char i,error=0;

  DATA=0;
  for (i=0x80;i>0;i>>=1)             //shift bit for masking
  { if (i & value) DATA1;          //masking value with i , write to SENSI-BUS
    else DATA0;                        
	_asm
	  DA A
	  DA A              ; 500ns delay, 20 cycles for a 40Mhz processor
          DA A
          DA A
          DA A
        _endasm;
    SCK=1;                          //clk for SENSI-BUS
	_asm
	  DA A
	  DA A              ; 500ns delay, 20 cycles for a 40Mhz processor
          DA A
          DA A
          DA A
        _endasm;
    SCK=0;
  }
  DATA1;                           //release DATA-line
  SCK=1;                            //clk #9 for ack 
  error=DATA;                       //check ack (DATA will be pulled down by SHT11)
  SCK=0;        
  return error;                     //error=1 in case of no acknowledge
}

//----------------------------------------------------------------------------------
char s_read_byte(unsigned char ack)
//----------------------------------------------------------------------------------
// reads a byte form the Sensibus and gives an acknowledge in case of "ack=1" 
{ 
unsigned char i,val=0;
  
  DATA=0;
  DATA1;                           //release DATA-line
  for (i=0x80;i>0;i/=2)             //shift bit for masking
  {
    SCK=1;                          //clk for SENSI-BUS
	_asm
	  DA A
	  DA A              ; 500ns delay, 20 cycles for a 40Mhz processor
          DA A
          DA A
          DA A
        _endasm;
    if (DATA) val=(val | i);        //read bit  
    SCK=0;  					 
	_asm
	  DA A
	  DA A              ; 500ns delay, 20 cycles for a 40Mhz processor
          DA A
          DA A
          DA A
        _endasm;
  }
  if(ack)                        //in case of "ack==1" pull down DATA-Line
    DATA0;
  SCK=1;                            //clk #9 for ack
	_asm
	  DA A
	  DA A              ; 500ns delay, 20 cycles for a 40Mhz processor
          DA A
          DA A
          DA A
        _endasm;
  SCK=0;						    
  DATA1;                           //release DATA-line
  return val;
}

//----------------------------------------------------------------------------------
void s_transstart(void)
//----------------------------------------------------------------------------------
// generates a transmission start 
//       _____         ________
// DATA:      |_______|
//           ___     ___
// SCK : ___|   |___|   |______
{
  DATA=0;
   DATA1; SCK=0;                   //Initial state
	_asm
	  DA A
	  DA A              ; 500ns delay, 20 cycles for a 40Mhz processor
          DA A
          DA A
          DA A
        _endasm;
   SCK=1;
	_asm
	  DA A
	  DA A              ; 300ns delay, 12 cycles for a 40Mhz processor
          DA A
        _endasm;
   DATA0;
	_asm
	  DA A
	  DA A              ; 300ns delay, 12 cycles for a 40Mhz processor
          DA A
        _endasm;
   SCK=0;  
	_asm
	  DA A
	  DA A              ; 500ns delay, 20 cycles for a 40Mhz processor
          DA A
          DA A
          DA A
        _endasm;
   SCK=1;
	_asm
	  DA A
	  DA A              ; 300ns delay, 12 cycles for a 40Mhz processor
          DA A
        _endasm;
   DATA1;		   
	_asm
	  DA A
	  DA A              ; 300ns delay, 12 cycles for a 40Mhz processor
          DA A
        _endasm;
   SCK=0;		   
}

//----------------------------------------------------------------------------------
void s_connectionreset(void)
//----------------------------------------------------------------------------------
// communication reset: DATA-line=1 and at least 9 SCK cycles followed by transstart
//       _____________________________________________________         ________
// DATA:                                                      |_______|
//          _    _    _    _    _    _    _    _    _        ___     ___
// SCK : __| |__| |__| |__| |__| |__| |__| |__| |__| |______|   |___|   |______
{  
  unsigned char i; 

  DATA1; SCK=0;                    //Initial state
  for(i=0;i<9;i++)                  //9 SCK cycles
  {
    SCK=1;
	_asm
	  DA A
	  DA A              ; 500ns delay, 20 cycles for a 40Mhz processor
          DA A
          DA A
          DA A
        _endasm;
    SCK=0;
	_asm
	  DA A
	  DA A              ; 500ns delay, 20 cycles for a 40Mhz processor
          DA A
          DA A
          DA A
        _endasm;

  }
  s_transstart();                   //transmission start
}

//----------------------------------------------------------------------------------
char s_softreset(void)
//----------------------------------------------------------------------------------
// resets the sensor by a softreset 
{ 
  unsigned char error=0;  
  s_connectionreset();              //reset communication
  error+=s_write_byte(RESET);       //send RESET-command to sensor
  return error;                     //error=1 in case of no response form the sensor
}

//----------------------------------------------------------------------------------
char s_read_statusreg(unsigned char *p_value, unsigned char *p_checksum)
//----------------------------------------------------------------------------------
// reads the status register with checksum (8-bit)
{ 
  unsigned char error=0;
  s_transstart();                   //transmission start
  error=s_write_byte(STATUS_REG_R); //send command to sensor
  *p_value=s_read_byte(ACK);        //read status register (8-bit)
  *p_checksum=s_read_byte(noACK);   //read checksum (8-bit)  
  return error;                     //error=1 in case of no response form the sensor
}

//----------------------------------------------------------------------------------
char s_write_statusreg(unsigned char *p_value)
//----------------------------------------------------------------------------------
// writes the status register with checksum (8-bit)
{ 
  unsigned char error=0;
  s_transstart();                   //transmission start
  error+=s_write_byte(STATUS_REG_W);//send command to sensor
  error+=s_write_byte(*p_value);    //send value of status register
  return error;                     //error>=1 in case of no response form the sensor
}
 							   
//----------------------------------------------------------------------------------
char s_measure_start(unsigned char mode)
//----------------------------------------------------------------------------------
// starts a measurement (humidity/temperature) with checksum
{ 
  unsigned char error=0;
  
    s_transstart();                   //transmission start
    switch(mode){                     //send command to sensor
    case TEMP: error+=s_write_byte(MEASURE_TEMP); break;
    case HUMI: error+=s_write_byte(MEASURE_HUMI); break;
    default: break;	 
    }
    return(error);
}

char s_measure_get(unsigned char *p_value, unsigned char *p_checksum)
//----------------------------------------------------------------------------------
    // call until it returns 0 (handle timeout as you wish)
    
{ 
    if(DATA==0) { // sensor has finished the measurement
      *(p_value+1)=s_read_byte(ACK);    //read the first byte (MSB)
      *(p_value)=s_read_byte(ACK);    //read the second byte (LSB)
      // beware of endianness !!!
      *p_checksum =s_read_byte(noACK);  //read checksum
      return(0);
    }
    return(1);
}


